@@ -1,8 +1,12 @@
+import os
+import secrets
 from pathlib import Path
-from typing import List, Optional, Pattern
+from typing import Callable, Dict, List, Optional, Pattern, Type, Union
 from urllib.parse import urlparse
 
 from pydantic import BaseSettings as PydanticBaseSettings, validator
+from starlette.applications import Starlette
+from starlette.middleware import Middleware
 from starlette.routing import Route
 from uvicorn.importer import import_from_string
 
@@ -34,6 +38,8 @@ class BaseSettings(PydanticBaseSettings):
     dev_mode: bool = False
     asgi_path: str = 'foxglove.asgi:app'
     routes: Optional[str] = None
+    middleware: Optional[str] = None
+    exception_handlers: Optional[str] = None
     web_workers: Optional[int] = None
 
     worker_func: Optional[str] = None
@@ -51,6 +57,10 @@ class BaseSettings(PydanticBaseSettings):
     redis_settings: Optional[RedisSettings] = redis_settings_default
     port: int = 8000
 
+    # secrets.token_hex() is used to avoid a public default value ever being used in production
+    secret_key: str = secrets.token_hex()
+    cookie_name: str = 'foxglove'
+
     locale: Optional[str] = None
 
     http_client_timeout = 10
@@ -75,6 +85,49 @@ class BaseSettings(PydanticBaseSettings):
         routes = self.routes or f'{self.__module__}:routes'
         return import_from_string(routes)
 
+    def get_middleware(self) -> List[Middleware]:
+        if self.middleware:
+            return import_from_string(self.middleware)
+        elif self.pg_dsn:
+            from .db import PgMiddleware
+
+            return [Middleware(PgMiddleware)]
+        else:
+            return []
+
+    def get_exception_handlers(self) -> Dict[Union[int, Type[Exception]], Callable]:
+        if self.exception_handlers:
+            return import_from_string(self.exception_handlers)
+        else:
+            from .exceptions import HttpRedirect, redirect_handler
+
+            return {HttpRedirect: redirect_handler}
+
+    def create_app(self) -> Starlette:
+        routes = list(self.get_routes())
+        if self.dev_mode:
+            foxglove_root_path = os.environ.get('foxglove_root_path')
+
+            from .devtools import reload_endpoint
+
+            if foxglove_root_path:
+                routes += reload_endpoint(foxglove_root_path)
+            else:
+                raise RuntimeError(
+                    'dev_mode enabled but "foxglove_root_path" not found, can\'t add the reload endpoint'
+                )
+
+        from .main import glove
+
+        return Starlette(
+            debug=self.dev_mode,
+            routes=routes,
+            middleware=list(self.get_middleware()),
+            exception_handlers=dict(self.get_exception_handlers()),
+            on_startup=[glove.startup],
+            on_shutdown=[glove.shutdown],
+        )
+
     @property
     def _pg_dsn_parsed(self):
         return urlparse(self.pg_dsn)
